<?php


namespace PaymentPlugins\WooCommerce\PPCP;


use PaymentPlugins\WooCommerce\PPCP\Admin\Settings\AdvancedSettings;
use PaymentPlugins\WooCommerce\PPCP\Payments\Gateways\AbstractGateway;

class OrderStatusController {

	private $advanced_settings;

	public function __construct( AdvancedSettings $advanced_settings ) {
		$this->advanced_settings = $advanced_settings;
		$this->initialize();
	}

	private function initialize() {
		add_action( 'woocommerce_order_status_completed', [ $this, 'handle_status_completed' ], 10, 2 );
		add_action( 'woocommerce_order_status_cancelled', [ $this, 'handle_status_cancelled' ], 10, 2 );
	}

	/**
	 * @param           $order_id
	 * @param \WC_Order $order
	 */
	public function handle_status_completed( $order_id, \WC_Order $order ) {
		if ( $this->advanced_settings->is_capture_on_complete() ) {
			$payment_method = $this->get_payment_gateway_from_order( $order );
			if ( $payment_method && $payment_method instanceof AbstractGateway ) {
				if ( ! $payment_method->payment_handler->is_processing( [ 'capture', 'payment' ] ) ) {
					$payment_method->payment_handler->set_processing( 'capture' );
					$payment_method->payment_handler->process_capture( $order );
				}
			}
		}
	}

	/**
	 * @param           $order_id
	 * @param \WC_Order $order
	 */
	public function handle_status_cancelled( $order_id, \WC_Order $order ) {
		if ( $this->advanced_settings->is_refund_on_cancel() ) {
			$payment_method = $this->get_payment_gateway_from_order( $order );
			if ( $payment_method && $payment_method instanceof AbstractGateway ) {
				if ( ! $payment_method->payment_handler->is_processing( 'void' ) ) {
					$payment_method->payment_handler->set_processing( 'void' );
					$payment_method->payment_handler->process_void( $order );
				}
			}
		}
	}

	private function get_payment_gateway_from_order( \WC_Order $order ) {
		$payment_methods = WC()->payment_gateways()->payment_gateways();
		$payment_method  = $order->get_payment_method();

		return isset( $payment_methods[ $payment_method ] ) ? $payment_methods[ $payment_method ] : null;
	}

}